@@ -1,3 +1,4 @@
+const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const GLib = imports.gi.GLib;
@@ -200,6 +201,7 @@ class GroupedWindowListApplet extends Applet.Applet {
                 pos: -1,
                 isForeign: null,
             },
+            appletActor: null,
             appletReady: false,
             willUnmount: false,
             settings: {},
@@ -285,6 +287,10 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.setAllowedLayout(Applet.AllowedLayout.BOTH);
         Gettext.bindtextdomain(metadata.uuid, GLib.get_home_dir() + '/.local/share/locale');
 
+        this.actor.set_style_class_name('grouped-window-list-box');
+        this.state.set({appletActor: this.actor});
+        this.on_orientation_changed(null);
+
         this.getAutoStartApps();
         this.onSwitchWorkspace = throttle(this.onSwitchWorkspace, 35, false); //Note: causes a 35ms delay in execution
         this.signals.connect(this.actor, 'scroll-event', (c, e) => this.handleScroll(e));
@@ -292,7 +298,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.signals.connect(global.window_manager, 'switch-workspace', (...args) => this.onSwitchWorkspace(...args));
         this.signals.connect(global.screen, 'workspace-removed', (...args) => this.onWorkspaceRemoved(...args));
         this.signals.connect(global.screen, 'window-monitor-changed', (...args) => this.onWindowMonitorChanged(...args));
-        this.signals.connect(global.screen, 'monitors-changed', (...args) => this.on_applet_instances_changed(...args));
+        this.signals.connect(Main.panelManager, 'monitors-changed', (...args) => this.on_applet_instances_changed(...args));
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', (...args) => this.onWindowSkipTaskbarChanged(...args));
         this.signals.connect(global.display, 'window-marked-urgent', (...args) => this.updateAttentionState(...args));
         this.signals.connect(global.display, 'window-demands-attention', (...args) => this.updateAttentionState(...args));
@@ -401,10 +407,12 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     on_orientation_changed(orientation) {
-        this.state.set({
-            orientation: orientation,
-            isHorizontal: orientation === St.Side.TOP || orientation === St.Side.BOTTOM
-        });
+        if (orientation) {
+            this.state.set({
+                orientation: orientation,
+                isHorizontal: orientation === St.Side.TOP || orientation === St.Side.BOTTOM
+            });
+        }
         if (this.state.isHorizontal) {
             this.actor.remove_style_class_name('vertical');
         } else {
@@ -608,7 +616,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         each(this.appLists, (workspace) => {
             each(workspace.appList, (appGroup) => {
                 if (titleDisplay === TitleDisplay.Focused) {
-                    appGroup.hideLabel(false);
+                    appGroup.hideLabel();
                 }
                 appGroup.handleTitleDisplayChange();
             });
@@ -672,6 +680,9 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     handleScroll(e, sourceFromAppGroup) {
+        if (e?.get_scroll_direction() == Clutter.ScrollDirection.SMOOTH)
+            return;
+
         if( (this.state.settings.thumbnailScrollBehavior) || (this.state.settings.scrollBehavior === 2) ||
             (this.state.settings.leftClickAction === 3 && this.state.settings.scrollBehavior !== 3
              && !e && sourceFromAppGroup)  ||
@@ -911,14 +922,16 @@ class GroupedWindowListApplet extends Applet.Applet {
         for (let i = 0; i < this.appLists.length; i++) {
             let workspaceIndex = this.appLists[i].metaWorkspace.index();
             if (workspaceIndex === -1) {
-                this.appLists[i].destroy();
-                this.appLists[i] = null;
+                if (this.appLists[i] != null) {
+                    this.appLists[i].destroy();
+                    this.appLists[i] = null;
+                }
                 removedLists.push(i);
             } else {
                 this.appLists[i].index = workspaceIndex;
             }
         }
-        for (let i = 0; i < removedLists.length; i++) {
+        for (let i = removedLists.length - 1; i >= 0; i--) {
             this.appLists.splice(removedLists[i], 1);
         }
         this.state.set({currentWs: global.screen.get_active_workspace_index()});
