@@ -74,7 +74,7 @@ class ControlButton {
     }
 
     setEnabled(status) {
-        this.button.change_style_pseudo_class("disabled", !status);
+        this.button.change_style_pseudo_class("insensitive", !status);
         this.button.can_focus = status;
         this.button.reactive = status;
     }
@@ -173,9 +173,12 @@ class VolumeSlider extends PopupMenu.PopupSliderMenuItem {
     }
 
     _onKeyPressEvent(actor, event) {
-        let key = event.get_key_symbol();
+        const key = event.get_key_symbol();
         if (key === Clutter.KEY_Right || key === Clutter.KEY_Left) {
             let delta = key === Clutter.KEY_Right ? VOLUME_ADJUSTMENT_STEP : -VOLUME_ADJUSTMENT_STEP;
+            if (St.Widget.get_default_direction() === St.TextDirection.RTL)
+                delta = -delta;
+
             this._value = Math.max(0, Math.min(this._value + delta/this.applet._volumeMax*this.applet._volumeNorm, 1));
             this._slider.queue_repaint();
             this.emit('value-changed', this._value);
@@ -242,6 +245,7 @@ class VolumeSlider extends PopupMenu.PopupSliderMenuItem {
 class Seeker extends Slider.Slider {
     constructor(mediaServerPlayer, props, playerName) {
         super(0, true);
+        this.actor.set_direction(St.TextDirection.LTR); //Do not invert on RTL layout
 
         this.canSeek = true;
         this.status = 'Stopped';
@@ -502,7 +506,11 @@ class Player extends PopupMenu.PopupMenuSection {
         // Player info
         this._playerBox = new St.BoxLayout();
         this.playerIcon = new St.Icon({icon_type: St.IconType.SYMBOLIC, style_class: "popup-menu-icon"});
-        this.playerLabel = new St.Label({ y_expand: true, y_align: Clutter.ActorAlign.CENTER, x_expand: true });
+        this.playerLabel = new St.Label({
+            y_expand: true, y_align: Clutter.ActorAlign.CENTER,
+            x_expand: true, x_align: Clutter.ActorAlign.START
+        });
+
         this._playerBox.add_actor(this.playerIcon);
         this._playerBox.add_actor(this.playerLabel);
 
@@ -526,6 +534,9 @@ class Player extends PopupMenu.PopupMenuSection {
         this.cover = new St.Icon({icon_name: "media-optical", icon_size: 300, icon_type: St.IconType.FULLCOLOR});
         this.coverBox.add_actor(this.cover);
 
+        this._cover_load_handle = 0;
+        this._cover_path = null;
+
         // Track info (artist + title)
         this._artist = _("Unknown Artist");
         this._album = _("Unknown Album");
@@ -563,7 +574,11 @@ class Player extends PopupMenu.PopupMenuSection {
                                              _("Next"),
                                              () => this._mediaServerPlayer.NextRemote());
         this.trackInfo.add_actor(trackControls);
+
         this.controls = new St.BoxLayout();
+        if(St.Widget.get_default_direction () === St.TextDirection.RTL)
+            this.controls.set_pack_start(true)
+
         this.controls.add_actor(this._prevButton.getActor());
         this.controls.add_actor(this._playButton.getActor());
         this.controls.add_actor(this._stopButton.getActor());
@@ -689,6 +704,8 @@ class Player extends PopupMenu.PopupMenuSection {
                 default:
                     this._artist = _("Unknown Artist");
             }
+            // make sure artist isn't empty
+            if (!this._artist) this._artist = _("Unknown Artist");
         }
         else
             this._artist = _("Unknown Artist");
@@ -709,9 +726,6 @@ class Player extends PopupMenu.PopupMenuSection {
         let change = false;
         if (metadata["mpris:artUrl"]) {
             let artUrl = metadata["mpris:artUrl"].unpack();
-            if ( this._name.toLowerCase() === "spotify" ) {
-                artUrl = artUrl.replace("open.spotify.com", "i.scdn.co");
-            }
             if (this._trackCoverFile != artUrl) {
                 this._trackCoverFile = artUrl;
                 change = true;
@@ -728,10 +742,31 @@ class Player extends PopupMenu.PopupMenuSection {
             if (this._trackCoverFile) {
                 let cover_path = "";
                 if (this._trackCoverFile.match(/^http/)) {
-                    if(!this._trackCoverFileTmp)
+                    if (!this._trackCoverFileTmp)
                         this._trackCoverFileTmp = Gio.file_new_tmp('XXXXXX.mediaplayer-cover')[0];
                     Util.spawn_async(['wget', this._trackCoverFile, '-O', this._trackCoverFileTmp.get_path()], () => this._onDownloadedCover());
                 }
+                else if (this._trackCoverFile.match(/data:image\/(png|jpeg);base64,/)) {
+                    if (!this._trackCoverFileTmp)
+                        this._trackCoverFileTmp = Gio.file_new_tmp('XXXXXX.mediaplayer-cover')[0];
+                    const cover_base64 = this._trackCoverFile.split(',')[1];
+                    const base64_decode = data => new Promise(resolve => resolve(GLib.base64_decode(data)));
+                    if (!cover_base64) {
+                        return;
+                    }
+                    base64_decode(cover_base64)
+                    .then(decoded => {
+                        this._trackCoverFileTmp.replace_contents(
+                            decoded,
+                            null,
+                            false,
+                            Gio.FileCreateFlags.REPLACE_DESTINATION,
+                            null
+                        );
+                        return this._trackCoverFileTmp.get_path();
+                    })
+                    .then(path => this._showCover(path));
+                }
                 else {
                     cover_path = decodeURIComponent(this._trackCoverFile);
                     cover_path = cover_path.replace("file://", "");
@@ -813,22 +848,32 @@ class Player extends PopupMenu.PopupMenuSection {
     }
 
     _showCover(cover_path) {
-        this.coverBox.remove_actor(this.cover);
         if (! cover_path || ! GLib.file_test(cover_path, GLib.FileTest.EXISTS)) {
             this.cover = new St.Icon({style_class: 'sound-player-generic-coverart', important: true, icon_name: "media-optical", icon_size: 300, icon_type: St.IconType.FULLCOLOR});
             cover_path = null;
         }
         else {
-            if (this._applet.keepAlbumAspectRatio) {
-                this.cover = new Clutter.Texture({width: 300, keep_aspect_ratio: true, filter_quality: 2, filename: cover_path});
-            }
-            else {
-                this.cover = new Clutter.Texture({width: 300, height: 300, keep_aspect_ratio: false, filter_quality: 2, filename: cover_path});
-            }
+            this._cover_path = cover_path;
+            this._cover_load_handle = St.TextureCache.get_default().load_image_from_file_async(cover_path, 300, 300, this._on_cover_loaded.bind(this));
         }
+    }
+
+    _on_cover_loaded(cache, handle, actor) {
+        if (handle !== this._cover_load_handle) {
+            // Maybe a cover image load stalled? Make sure our requests match the callback.
+            return;
+        }
+
+        this.coverBox.remove_actor(this.cover);
+
+        // Make sure any oddly-shaped album art doesn't affect the height of the applet popup
+        // (and move the player controls as a result).
+        actor.margin_bottom = 300 - actor.height;
+
+        this.cover = actor;
         this.coverBox.add_actor(this.cover);
         this.coverBox.set_child_below_sibling(this.cover, this.trackInfo);
-        this._applet.setAppletTextIcon(this, cover_path);
+        this._applet.setAppletTextIcon(this, this._cover_path);
     }
 
     onSettingsChanged() {
@@ -877,7 +922,6 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
         this.settings.bind("horizontalScroll", "horizontalScroll")
         this.settings.bind("showalbum", "showalbum", this.on_settings_changed);
         this.settings.bind("truncatetext", "truncatetext", this.on_settings_changed);
-        this.settings.bind("keepAlbumAspectRatio", "keepAlbumAspectRatio", this.on_settings_changed);
         this.settings.bind("hideSystray", "hideSystray", function() {
             if (this.hideSystray) this.registerSystrayIcons();
             else this.unregisterSystrayIcons();
@@ -891,9 +935,13 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
 
         this.settings.bind("_knownPlayers", "_knownPlayers");
         if (this.hideSystray) this.registerSystrayIcons();
-        
+
         this.settings.bind("keyOpen", "keyOpen", this._setKeybinding);
 
+        this.settings.bind("tooltipShowVolume", "tooltipShowVolume", this.on_settings_changed);
+        this.settings.bind("tooltipShowPlayer", "tooltipShowPlayer", this.on_settings_changed);
+        this.settings.bind("tooltipShowArtistTitle", "tooltipShowArtistTitle", this.on_settings_changed);
+
         this.menuManager = new PopupMenu.PopupMenuManager(this);
         this.menu = new Applet.AppletPopupMenu(this, orientation);
         this.menuManager.addMenu(this.menu);
@@ -978,6 +1026,7 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
         this.mute_in_switch = new PopupMenu.PopupSwitchIconMenuItem(_("Mute input"), false, "microphone-sensitivity-none", St.IconType.SYMBOLIC);
         this._applet_context_menu.addMenuItem(this.mute_out_switch);
         this._applet_context_menu.addMenuItem(this.mute_in_switch);
+        this.mute_in_switch.actor.hide();
 
         this._applet_context_menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
@@ -1021,7 +1070,7 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
 
         this._sound_settings.connect("changed::" + MAXIMUM_VOLUME_KEY, () => this._on_sound_settings_change());
     }
-    
+
     _setKeybinding() {
         Main.keybindingManager.addHotKey("sound-open-" + this.instance_id, this.keyOpen, Lang.bind(this, this._openMenu));
     }
@@ -1063,7 +1112,7 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
     on_applet_clicked(event) {
         this._openMenu();
     }
-    
+
     _openMenu() {
         this.menu.toggle();
     }
@@ -1131,7 +1180,9 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
 
         this._applet_tooltip.show();
 
-        this._notifyVolumeChange(this._output);
+        if (event.get_scroll_direction() != Clutter.ScrollDirection.SMOOTH) {
+            this._notifyVolumeChange(this._output);
+        }
     }
 
     _onButtonPressEvent (actor, event) {
@@ -1221,7 +1272,12 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
     setAppletText(player) {
         let title_text = "";
         if (this.showtrack && player && player._playerStatus == 'Playing') {
-            title_text = player._title + ' - ' + player._artist;
+            if (player._artist == "Unknown Artist") {
+                title_text = player._title;
+            }
+            else {
+                title_text = player._title + ' - ' + player._artist;
+            }
             if (this.truncatetext < title_text.length) {
                 title_text = title_text.substr(0, this.truncatetext) + "...";
             }
@@ -1230,10 +1286,33 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
     }
 
     setAppletTextIcon(player, icon) {
+        this.player = player;
         if (player && player._owner != this._activePlayer)
             return;
         this.setAppletIcon(player, icon);
         this.setAppletText(player);
+        this.setAppletTooltip();
+    }
+
+    setAppletTooltip() {
+        let tooltips = [];
+        if (this.tooltipShowVolume) {
+            tooltips.push(_("Volume") + ": " + this.volume);
+        }
+        if (this.player && this.player._owner == this._activePlayer) {
+            if (this.tooltipShowPlayer) {
+                tooltips.push(this.player._name + " - " + _(this.player._playerStatus));
+            }
+            if (this.tooltipShowArtistTitle) {
+                if (this.player._artist != _("Unknown Artist")) {
+                    tooltips.push(this.player._artist);
+                }
+                if (this._title != _("Unknown Title")) {
+                    tooltips.push(this.player._title);
+                }
+            }
+        }
+        this.set_applet_tooltip(tooltips.join("\n"));
     }
 
     _isInstance(busName) {
@@ -1349,7 +1428,7 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
         this._launchPlayerItem = new PopupMenu.PopupSubMenuMenuItem(_("Launch player"));
         this.menu.addMenuItem(this._launchPlayerItem);
         this._updateLaunchPlayer();
-        
+
         // The list to use when switching between active players
         this._chooseActivePlayerItem = new PopupMenu.PopupSubMenuMenuItem(_("Choose player controls"));
         this._chooseActivePlayerItem.actor.hide();
@@ -1433,7 +1512,7 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
     }
 
     _notifyVolumeChange(stream) {
-        Main.soundManager.playVolume('volume', stream.decibel);
+        Main.soundManager.play('volume');
     }
 
     _mutedChanged(object, param_spec, property) {
@@ -1447,7 +1526,8 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
     _outputValuesChanged(actor, iconName, percentage) {
         this.setIcon(iconName, "output");
         this.mute_out_switch.setIconSymbolicName(iconName);
-        this.set_applet_tooltip(_("Volume") + ": " + percentage);
+        this.volume = percentage;
+        this.setAppletTooltip();
     }
 
     _inputValuesChanged(actor, iconName) {
@@ -1559,8 +1639,10 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
         } else if (stream instanceof Cvc.MixerSourceOutput) {
             //for source outputs, only show the input section
             this._streams.push({id: id, type: "SourceOutput"});
-            if (this._recordingAppsNum++ === 0)
+            if (this._recordingAppsNum++ === 0) {
                 this._inputSection.actor.show();
+                this.mute_in_switch.actor.show();
+            }
         }
     }
 
@@ -1576,8 +1658,10 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
                     if (this._outputApplicationsMenu.menu.numMenuItems === 0)
                         this._outputApplicationsMenu.actor.hide();
                 } else if (stream.type === "SourceOutput") {
-                    if(--this._recordingAppsNum === 0)
+                    if(--this._recordingAppsNum === 0) {
                         this._inputSection.actor.hide();
+                        this.mute_in_switch.actor.hide();
+                    }
                 }
                 this._streams.splice(i, 1);
                 break;
@@ -1587,15 +1671,15 @@ class CinnamonSoundApplet extends Applet.TextIconApplet {
 
     registerSystrayIcons() {
         for (let i = 0; i < players_with_seek_support.length; i++) {
-            Main.systrayManager.registerRole(players_with_seek_support[i], this.metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement(players_with_seek_support[i], this.metadata.uuid);
         }
         for (let i = 0; i < players_without_seek_support.length; i++) {
-            Main.systrayManager.registerRole(players_without_seek_support[i], this.metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement(players_without_seek_support[i], this.metadata.uuid);
         }
     }
 
     unregisterSystrayIcons() {
-        Main.systrayManager.unregisterId(this.metadata.uuid);
+        Main.systrayManager.unregisterTrayIconReplacement(this.metadata.uuid);
     }
 }
 
