@@ -16,7 +16,6 @@ const Tweener = imports.ui.tweener;
 
 const WindowUtils = imports.misc.windowUtils;
 
-const POPUP_APPICON_SIZE = 96;
 const POPUP_SCROLL_TIME = 0.10; // seconds
 const POPUP_DELAY_TIMEOUT = 150; // milliseconds
 const POPUP_FADE_OUT_TIME = 0.1; // seconds
@@ -24,11 +23,11 @@ const POPUP_FADE_OUT_TIME = 0.1; // seconds
 const APP_ICON_HOVER_TIMEOUT = 200; // milliseconds
 
 const THUMBNAIL_DEFAULT_SIZE = 256;
-const THUMBNAIL_POPUP_TIME = 180; // milliseconds
+const THUMBNAIL_POPUP_TIME = 300; // milliseconds
 const THUMBNAIL_FADE_TIME = 0.1; // seconds
 
-const PREVIEW_DELAY_TIMEOUT = 180; // milliseconds
-var PREVIEW_SWITCHER_FADEOUT_TIME = 0.5; // seconds
+const PREVIEW_DELAY_TIMEOUT = 0; // milliseconds
+var PREVIEW_SWITCHER_FADEOUT_TIME = 0.2; // seconds
 
 const iconSizes = [96, 64, 48, 32, 22];
 
@@ -76,6 +75,9 @@ ClassicSwitcher.prototype = {
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
         
+        this._applist_act_id = 0;
+        this._applist_enter_id = 0;
+
         // Need to force an allocation so we can figure out whether we
         // need to scroll when selecting
         this.actor.opacity = 0;
@@ -141,8 +143,7 @@ ClassicSwitcher.prototype = {
     },
 
     _show: function() {
-        let panels = Main.getPanels();
-        panels.forEach(function(panel) { panel.actor.set_reactive(false); });
+        Main.panelManager.panels.forEach(function(panel) { panel.actor.set_reactive(false); });
         
         this.actor.opacity = 255;
         this._initialDelayTimeoutId = 0;
@@ -157,8 +158,7 @@ ClassicSwitcher.prototype = {
         }
 
         // panels
-        let panels = Main.getPanels();
-        panels.forEach(function(panel) { panel.actor.set_reactive(true); });
+        Main.panelManager.panels.forEach(function(panel) { panel.actor.set_reactive(true); });
 
         Tweener.addTween(this.actor, { opacity: 0,
             time: POPUP_FADE_OUT_TIME,
@@ -177,6 +177,14 @@ ClassicSwitcher.prototype = {
             return;
         
         if (this._appList) {
+            if (this._applist_act_id !== 0) {
+                this._appList.disconnect(this._applist_act_id);
+                this._applist_act_id = 0;
+            }
+            if (this._applist_enter_id !== 0) {
+                this._appList.disconnect(this._applist_enter_id);
+                this._applist_enter_id = 0;
+            }
             this._clearPreview();
             this._destroyThumbnails();
             this.actor.remove_actor(this._appList.actor);
@@ -187,10 +195,11 @@ ClassicSwitcher.prototype = {
         if (!this._iconsEnabled && !this._thumbnailsEnabled) {
             this._appList.actor.hide();
         }
-        this._appList.connect('item-activated', Lang.bind(this, this._appActivated));
-        this._appList.connect('item-entered', Lang.bind(this, this._appEntered));
+        this._applist_act_id = this._appList.connect('item-activated', Lang.bind(this, this._appActivated));
+        this._applist_enter_id = this._appList.connect('item-entered', Lang.bind(this, this._appEntered));
         
         this._appIcons = this._appList.icons;
+        this.actor.get_allocation_box();
     },
 
     _selectNext: function() {
@@ -219,8 +228,8 @@ ClassicSwitcher.prototype = {
     _setCurrentWindow: function(window) {
         this._appList.highlight(this._currentIndex, false);
         this._doWindowPreview();
-        
         this._destroyThumbnails();
+        
         if (this._thumbnailTimeoutId != 0) {
             Mainloop.source_remove(this._thumbnailTimeoutId);
             this._thumbnailTimeoutId = 0;
@@ -229,18 +238,36 @@ ClassicSwitcher.prototype = {
         if (this._showArrows) {
             this._thumbnailTimeoutId = Mainloop.timeout_add(
                 THUMBNAIL_POPUP_TIME, Lang.bind(this, function() {
+
                     if (!this._thumbnails)
                         this._createThumbnails();
                     this._thumbnails.highlight(0, false);
+                    this._thumbnailTimeoutId = 0;
             }));
         }
     },
 
     _onDestroy: function() {
-        if (this._thumbnailTimeoutId)
+        if (this._appList !== null) {
+            if (this._applist_act_id > 0) {
+                this._appList.disconnect(this._applist_act_id);
+                this._applist_act_id = 0;
+            }
+
+            if (this._applist_enter_id > 0) {
+                this._appList.disconnect(this._applist_enter_id);
+                this._applist_enter_id = 0;
+            }
+        }
+
+        if (this._thumbnailTimeoutId != 0) {
             Mainloop.source_remove(this._thumbnailTimeoutId);
-        if (this._displayPreviewTimeoutId)
+            this._thumbnailTimeoutId = 0;
+        }
+        if (this._displayPreviewTimeoutId != 0) {
             Mainloop.source_remove(this._displayPreviewTimeoutId);
+            this._displayPreviewTimeoutId = 0;
+        }
     },
 
     _appActivated : function(appSwitcher, n) {
@@ -286,20 +313,21 @@ ClassicSwitcher.prototype = {
         // Use a cancellable timeout to avoid flickering effect when tabbing rapidly through the set.
         if (this._displayPreviewTimeoutId) {
             Mainloop.source_remove(this._displayPreviewTimeoutId);
+            this._displayPreviewTimeoutId = 0;
         }
         let delay = PREVIEW_DELAY_TIMEOUT;
         this._displayPreviewTimeoutId = Mainloop.timeout_add(delay, Lang.bind(this, this._showWindowPreview));
     },
     
     _showWindowPreview: function() {
-        this._displayPreviewTimeoutId = null;
+        this._displayPreviewTimeoutId = 0;
 
         let childBox = new Clutter.ActorBox();
 
         let lastClone = null;
         let previewClones = [];
         let window = this._windows[this._currentIndex];
-        let clones = WindowUtils.createWindowClone(window, 0, 0, true, false);
+        let clones = WindowUtils.createLiveWindowClone(window, 0, 0, true, false);
         for (let i = 0; i < clones.length; i++) {
             let clone = clones[i];
             previewClones.push(clone.actor);
@@ -335,36 +363,22 @@ ClassicSwitcher.prototype = {
             backdrop.opacity = 0;
             Tweener.addTween(backdrop,
                             { opacity: 255,
-                            time: PREVIEW_SWITCHER_FADEOUT_TIME,
+                            time: PREVIEW_SWITCHER_FADEOUT_TIME / 4,
                             transition: 'linear'
                             });
         }
     },
 
-    _timeoutPopupThumbnails: function() {
-        if (!this._thumbnails)
-            this._createThumbnails();
-        this._thumbnailTimeoutId = 0;
-        this._thumbnailsFocused = false;
-        return false;
-    },
-
     _destroyThumbnails : function() {
         if (!this._thumbnails) {
             return;
         }
         let thumbnailsActor = this._thumbnails.actor;
         this._thumbnails = null;
-        Tweener.addTween(thumbnailsActor,
-            { opacity: 0,
-                time: THUMBNAIL_FADE_TIME,
-                transition: 'easeOutQuad',
-                onComplete: Lang.bind(this, function() {
-                    this.actor.remove_actor(thumbnailsActor);
-                    thumbnailsActor.destroy();
-                    this.thumbnailsVisible = false;
-                })
-            });
+        this.actor.remove_actor(thumbnailsActor);
+        thumbnailsActor.destroy();
+        this.thumbnailsVisible = false;
+        
     },
 
     _createThumbnails : function() {
@@ -410,15 +424,9 @@ AppIcon.prototype = {
             if (window.minimized) {
                 this.label = new St.Label({ text: "[" + title + "]"});               
                 let contrast_effect = new Clutter.BrightnessContrastEffect();                
-                contrast_effect.set_brightness(-0.5, -0.5, -0.5);                         
+                contrast_effect.set_brightness_full(-0.5, -0.5, -0.5);
                 this._iconBin.add_effect(contrast_effect);                
             }
-            else if (window.tile_type == Meta.WindowTileType.TILED) {
-                this.label = new St.Label({ text: "|" + title });
-            }
-            else if (window.tile_type == Meta.WindowTileType.SNAPPED) {
-                this.label = new St.Label({ text: "||" + title });
-            }
             else {
                 this.label = new St.Label({ text: title });    
             }
@@ -435,8 +443,8 @@ AppIcon.prototype = {
 
     set_size: function(size) {
         if (this.showThumbnail){
-            this.icon = new St.Group();
-            let clones = WindowUtils.createWindowClone(this.window, size, size, true, true);
+            this.icon = new St.Widget();
+            let clones = WindowUtils.createLiveWindowClone(this.window, size * global.ui_scale, size * global.ui_scale, true, true);
             for (let i in clones) {
                 let clone = clones[i];
                 this.icon.add_actor(clone.actor);
@@ -447,11 +455,12 @@ AppIcon.prototype = {
             }
         } else {
             this.icon = this.app ?
-                this.app.create_icon_texture(size) :
+                this.app.create_icon_texture_for_window(size, this.window) :
                 new St.Icon({ icon_name: 'application-default-icon',
                               icon_type: St.IconType.FULLCOLOR,
                               icon_size: size });
         }
+        size *= global.ui_scale;
         this._iconBin.set_size(size, size);
         this._iconBin.child = this.icon;
     }
@@ -831,7 +840,7 @@ AppList.prototype = {
 
         for(let i =  0; i < iconSizes.length; i++) {
                 this._iconSize = iconSizes[i];
-                height = iconSizes[i] + iconSpacing;
+                height = (iconSizes[i] * global.ui_scale) + iconSpacing;
                 let w = height * this._items.length + totalSpacing;
                 if (w <= availWidth)
                         break;
@@ -839,7 +848,7 @@ AppList.prototype = {
 
         if (this._items.length == 1) {
             this._iconSize = iconSizes[0];
-            height = iconSizes[0] + iconSpacing;
+            height = (iconSizes[0] * global.ui_scale) + iconSpacing;
         }
 
         for(let i = 0; i < this.icons.length; i++) {
@@ -984,8 +993,8 @@ ThumbnailList.prototype = {
 
         for (let i = 0; i < this._thumbnailBins.length; i++) {
             let metaWindow = this._windows[i];
-            let container = new St.Group();
-            let clones = WindowUtils.createWindowClone(metaWindow, availHeight, availHeight, true, true);
+            let container = new St.Widget();
+            let clones = WindowUtils.createLiveWindowClone(metaWindow, availHeight, availHeight, true, true);
             for (let j = 0; j < clones.length; j++) {
               let clone = clones[j];
               container.add_actor(clone.actor);
@@ -1042,4 +1051,6 @@ function _drawArrow(area, side) {
 
     Clutter.cairo_set_source_color(cr, bodyColor);
     cr.fill();
+
+    cr.$dispose();
 }
