@@ -91,7 +91,7 @@ LayoutManager.prototype = {
 
         global.settings.connect("changed::enable-edge-flip", Lang.bind(this, this._onEdgeFlipChanged));
         global.settings.connect("changed::edge-flip-delay", Lang.bind(this, this._onEdgeFlipChanged));
-        global.screen.connect('monitors-changed', Lang.bind(this, this._monitorsChanged));
+        Meta.MonitorManager.get().connect('monitors-changed', Lang.bind(this, this._monitorsChanged));
     },
 
     _onEdgeFlipChanged: function(){
@@ -132,20 +132,22 @@ LayoutManager.prototype = {
     },
 
     _updateMonitors: function() {
-        let screen = global.screen;
-
         this.monitors = [];
-        let nMonitors = screen.get_n_monitors();
-        for (let i = 0; i < nMonitors; i++)
-            this.monitors.push(new Monitor(i, screen.get_monitor_geometry(i)));
+        let nMonitors = global.display.get_n_monitors();
+        for (let i = 0; i < nMonitors; i++) {
+            let rect = global.display.get_monitor_geometry(i);
+            let lmon = global.display.get_monitor_index_for_rect(rect);
 
-        this.primaryIndex = screen.get_primary_monitor();
+            this.monitors.push(new Monitor(lmon, rect));
+        }
+
+        this.primaryIndex = global.display.get_primary_monitor();
         this.primaryMonitor = this.monitors[this.primaryIndex];
     },
 
     _updateBoxes: function() {
         if (this.hotCornerManager)
-            this.hotCornerManager.updatePosition(this.primaryMonitor);
+            this.hotCornerManager.update();
         this._chrome._queueUpdateRegions();
     },
 
@@ -204,7 +206,7 @@ LayoutManager.prototype = {
 
     _startupAnimationComplete: function() {
         global.stage.show_cursor();
-        this._coverPane.destroy();
+        this.removeChrome(this._coverPane);
         this._coverPane = null;
         this._chrome.thawUpdateRegions();
 
@@ -375,6 +377,11 @@ LayoutManager.prototype = {
         return this._chrome.findMonitorIndexForActor(actor);
     },
 
+    findMonitorIndexAt: function(x, y) {
+        let [index, monitor] = this._chrome._findMonitorForRect(x, y, 1, 1)
+        return index;
+    },
+
     /**
      * isTrackingChrome:
      * @actor (Clutter.Actor): the actor to check
@@ -482,8 +489,8 @@ Chrome.prototype = {
             return;
         let actorData = this._trackedActors[i];
 
-        if (actorData.addToWindowgroup) global.window_group.remove_actor(actor);
-        else Main.uiGroup.remove_actor(actor);
+        if (actorData.addToWindowgroup) global.window_group.remove_child(actor);
+        else Main.uiGroup.remove_child(actor);
         this._untrackActor(actor);
     },
 
