@@ -11,7 +11,7 @@
  * @messageTray (MessageTray.MessageTray): The mesesage tray
  * @notificationDaemon (NotificationDaemon.NotificationDaemon): The notification daemon
  * @windowAttentionHandler (WindowAttentionHandler.WindowAttentionHandler): The window attention handle
- * @recorder (Cinnamon.Recorder): The recorder
+ * @screenRecorder (ScreenRecorder.ScreenRecorder): The recorder
  * @cinnamonDBusService (CinnamonDBus.Cinnamon): The cinnamon dbus object
  * @screenshotService (Screenshot.ScreenshotService): Implementation of gnome-shell's screenshot interface.
  * @modalCount (int): The number of modals "pushed"
@@ -22,7 +22,7 @@
  * @magnifier (Magnifier.Magnifier): The magnifier
  * @xdndHandler (XdndHandler.XdndHandler): The X DND handler
  * @statusIconDispatcher (StatusIconDispatcher.StatusIconDispatcher): The status icon dispatcher
- * @keyboard (Keyboard.Keyboard): The keyboard object
+ * @virtualKeyboard (VirtualKeyboard.Keyboard): The keyboard object
  * @layoutManager (Layout.LayoutManager): The layout manager.
  * \
  * All actors that are part of the Cinnamon UI ar handled by the layout
@@ -62,6 +62,7 @@
  * This is a container that contains all the desklets as childs. Its actor is
  * put between @global.bottom_window_group and @global.uiGroup.
  * @software_rendering (boolean): Whether software rendering is used
+ * @animations_enabled (boolean): Whether any effects or animations should be used.
  * @popup_rendering_actor (Clutter.Actor): The popup actor that is in the process of rendering
  * @xlet_startup_error (boolean): Whether there was at least one xlet that did
  * not manage to load
@@ -91,7 +92,7 @@ var AppletManager = imports.ui.appletManager;
 const SearchProviderManager = imports.ui.searchProviderManager;
 const DeskletManager = imports.ui.deskletManager;
 const ExtensionSystem = imports.ui.extensionSystem;
-const Keyboard = imports.ui.keyboard;
+const VirtualKeyboard = imports.ui.virtualKeyboard;
 const MessageTray = imports.ui.messageTray;
 const OsdWindow = imports.ui.osdWindow;
 const Overview = imports.ui.overview;
@@ -117,6 +118,8 @@ const Accessibility = imports.ui.accessibility;
 const ModalDialog = imports.ui.modalDialog;
 const {readOnlyError} = imports.ui.environment;
 const {installPolyfills} = imports.ui.overrides;
+const InputMethod = imports.misc.inputMethod;
+const ScreenRecorder = imports.ui.screenRecorder;
 
 var LAYOUT_TRADITIONAL = "traditional";
 var LAYOUT_FLIPPED = "flipped";
@@ -140,7 +143,7 @@ var a11yHandler = null;
 var messageTray = null;
 var notificationDaemon = null;
 var windowAttentionHandler = null;
-var recorder = null;
+var screenRecorder = null;
 var cinnamonDBusService = null;
 var screenshotService = null;
 var modalCount = 0;
@@ -149,7 +152,7 @@ var uiGroup = null;
 var magnifier = null;
 var xdndHandler = null;
 var statusIconDispatcher = null;
-var keyboard = null;
+var virtualKeyboard = null;
 var layoutManager = null;
 var themeManager = null;
 var keybindingManager = null;
@@ -162,6 +165,7 @@ var tracker = null;
 var settingsManager = null;
 var systrayManager = null;
 var wmSettings = null;
+var pointerTracker = null;
 
 var workspace_names = [];
 
@@ -169,6 +173,7 @@ var applet_side = St.Side.TOP; // Kept to maintain compatibility. Doesn't seem t
 var deskletContainer = null;
 
 var software_rendering = false;
+var animations_enabled = false;
 
 var popup_rendering_actor = null;
 
@@ -199,39 +204,6 @@ function setRunState(state) {
     }
 }
 
-function _initRecorder() {
-    let recorderSettings = new Gio.Settings({ schema_id: 'org.cinnamon.recorder' });
-
-    global.screen.connect('toggle-recording', function() {
-        if (recorder == null) {
-            recorder = new Cinnamon.Recorder({ stage: global.stage });
-        }
-
-        if (recorder.is_recording()) {
-            recorder.pause();
-            Meta.enable_unredirect_for_screen(global.screen);
-        } else {
-            // read the parameters from GSettings always in case they have changed
-            recorder.set_framerate(recorderSettings.get_int('framerate'));
-            recorder.set_filename('cinnamon-%d%u-%c.' + recorderSettings.get_string('file-extension'));
-            let pipeline = recorderSettings.get_string('pipeline');
-
-            if (layoutManager.monitors.length > 1) {
-                let {x, y, width, height} = layoutManager.primaryMonitor;
-                recorder.set_area(x, y, width, height);
-            }
-
-            if (!pipeline.match(/^\s*$/))
-                recorder.set_pipeline(pipeline);
-            else
-                recorder.set_pipeline(null);
-
-            Meta.disable_unredirect_for_screen(global.screen);
-            recorder.record();
-        }
-    });
-}
-
 function _addXletDirectoriesToSearchPath() {
     imports.searchPath.unshift(global.datadir);
     imports.searchPath.unshift(global.userdatadir);
@@ -253,9 +225,7 @@ function _addXletDirectoriesToSearchPath() {
 }
 
 function _initUserSession() {
-    _initRecorder();
-
-    global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT, false, 1, -1);
+    global.screen.override_workspace_layout(Meta.DisplayCorner.TOPLEFT, false, 1, -1);
 
     systrayManager = new Systray.SystrayManager();
 
@@ -299,10 +269,27 @@ function start() {
     let cinnamonStartTime = new Date().getTime();
 
     log("About to start Cinnamon");
-    if (GLib.getenv('CINNAMON_SOFTWARE_RENDERING')) {
-        log("ACTIVATING SOFTWARE RENDERING");
+
+    let backend = Meta.get_backend();
+
+    // Only cinnamon2d laucher will set CINNAMON_2D - this is deliberate by the user.
+    let cinnamon_2d = GLib.getenv("CINNAMON_2D") === true;
+    let live = false;
+
+    if (!backend.is_rendering_hardware_accelerated() || cinnamon_2d) {
         global.logError("Cinnamon Software Rendering mode enabled");
         software_rendering = true;
+
+        // We only warn if software_rendering is not of the user's volition.
+        if (!cinnamon_2d && GLib.file_test("/proc/cmdline", GLib.FileTest.EXISTS)) {
+            let content = Cinnamon.get_file_contents_utf8_sync("/proc/cmdline");
+            if (content.match("boot=casper") || content.match("boot=live")) {
+                // If we're in a live session, pretend we're using hardware rendering,
+                // so all animations end up being enabled.
+                software_rendering = false;
+                live = true;
+            }
+        }
     }
 
     // Chain up async errors reported from C
@@ -310,6 +297,8 @@ function start() {
 
     Gio.DesktopAppInfo.set_desktop_env('X-Cinnamon');
 
+    Clutter.get_default_backend().set_input_method(new InputMethod.InputMethod());
+
     cinnamonDBusService = new CinnamonDBus.CinnamonDBus();
     setRunState(RunState.STARTUP);
 
@@ -354,6 +343,9 @@ function start() {
 
     // Set up stage hierarchy to group all UI actors under one container.
     uiGroup = new Cinnamon.GenericContainer({ name: 'uiGroup' });
+    // see https://gitlab.gnome.org/GNOME/gnome-shell/-/commit/73d8c82640f6125cfbf80
+    this.uiGroup.set_flags(Clutter.ActorFlags.NO_LAYOUT);
+
     uiGroup.connect('allocate',
                     function (actor, box, flags) {
                         let children = uiGroup.get_children();
@@ -370,11 +362,7 @@ function start() {
                         let height = global.stage.height;
                         [alloc.min_size, alloc.natural_size] = [height, height];
                     });
-
-    global.reparentActor(global.background_actor, uiGroup);
     global.background_actor.hide();
-    global.reparentActor(global.bottom_window_group, uiGroup);
-    uiGroup.add_actor(deskletContainer.actor);
     global.reparentActor(global.window_group, uiGroup);
     global.reparentActor(global.overlay_group, uiGroup);
 
@@ -399,16 +387,15 @@ function start() {
 
     let startupAnimationEnabled = global.settings.get_boolean("startup-animation");
 
-    let do_animation = !global.session_running &&
-                        startupAnimationEnabled &&
-                       !GLib.getenv('CINNAMON_SOFTWARE_RENDERING') &&
-                       !GLib.getenv('CINNAMON_2D');
+    let do_startup_animation = !global.session_running &&
+                                startupAnimationEnabled &&
+                                !software_rendering;
 
-    if (do_animation) {
+    if (do_startup_animation) {
         layoutManager._prepareStartupAnimation();
     }
 
-    let pointerTracker = new PointerTracker.PointerTracker();
+    pointerTracker = new PointerTracker.PointerTracker();
     pointerTracker.setPosition(layoutManager.primaryMonitor.x + layoutManager.primaryMonitor.width/2,
         layoutManager.primaryMonitor.y + layoutManager.primaryMonitor.height/2);
 
@@ -424,7 +411,7 @@ function start() {
 
     wm = new imports.ui.windowManager.WindowManager();
     messageTray = new MessageTray.MessageTray();
-    keyboard = new Keyboard.Keyboard();
+    virtualKeyboard = new VirtualKeyboard.Keyboard();
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
     placesManager = new PlacesManager.PlacesManager();
@@ -432,12 +419,13 @@ function start() {
     magnifier = new Magnifier.Magnifier();
 
     layoutManager.init();
-    keyboard.init();
+    virtualKeyboard.init();
     overview.init();
     expo.init();
 
     _addXletDirectoriesToSearchPath();
     _initUserSession();
+    screenRecorder = new ScreenRecorder.ScreenRecorder();
 
     // Provide the bus object for gnome-session to
     // initiate logouts.
@@ -445,6 +433,11 @@ function start() {
 
     _startDate = new Date();
 
+    global.display.connect('restart', () => {
+        global.real_restart();
+        return true;
+    });
+
     global.stage.connect('captured-event', _stageEventHandler);
 
     global.log('loaded at ' + _startDate);
@@ -453,15 +446,27 @@ function start() {
     wmSettings = new Gio.Settings({schema_id: "org.cinnamon.desktop.wm.preferences"})
     workspace_names = wmSettings.get_strv("workspace-names");
 
+    wmSettings.connect("changed::workspace-names", function (settings, pspec) {
+        workspace_names = wmSettings.get_strv("workspace-names");
+    });
+
     global.display.connect('gl-video-memory-purged', loadTheme);
 
     try {
-        gpu_offload_supported = XApp.util_gpu_offload_supported()
+        gpu_offload_supported = Cinnamon.get_gpu_offload_supported()
     } catch (e) {
         global.logWarning("Could not check for gpu offload support - maybe xapps isn't up to date.");
         gpu_offload_supported = false;
     }
 
+    log(`GPU offload supported: ${gpu_offload_supported}`);
+
+    // We're ready for the session manager to move to the next phase
+    GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
+        Meta.register_with_session();
+        return GLib.SOURCE_REMOVE;
+    });
+
     Promise.all([
         AppletManager.init(),
         ExtensionSystem.init(),
@@ -472,13 +477,9 @@ function start() {
 
         a11yHandler = new Accessibility.A11yHandler();
 
-        if (software_rendering && !GLib.getenv('CINNAMON_2D')) {
-            if (GLib.file_test("/proc/cmdline", GLib.FileTest.EXISTS)) {
-                let content = Cinnamon.get_file_contents_utf8_sync("/proc/cmdline");
-                if (!content.match("boot=casper") && !content.match("boot=live")) {
-                    notifyCinnamon2d();
-                }
-            }
+        // We only warn if software_rendering is not of the user's volition.
+        if (software_rendering && !cinnamon_2d && !live) {
+            notifyCinnamon2d();
         }
 
         if (xlet_startup_error)
@@ -493,21 +494,19 @@ function start() {
         // until the event loop is uncontended and idle.
         // This helps to prevent us from running the animation
         // when the system is bogged down
-        if (do_animation) {
+        if (do_startup_animation) {
             let id = GLib.idle_add(GLib.PRIORITY_LOW, () => {
-                if (do_login_sound)
-                    soundManager.play_once_per_session('login');
                 layoutManager._doStartupAnimation();
                 return GLib.SOURCE_REMOVE;
             });
         } else {
             global.background_actor.show();
             setRunState(RunState.RUNNING);
-
-            if (do_login_sound)
-                soundManager.play_once_per_session('login');
         }
 
+        if (do_login_sound && !global.session_running)
+		    soundManager.play('login');
+
         // Disable panel edit mode when Cinnamon starts
         if (global.settings.get_boolean("panel-edit-mode")) {
             global.settings.set_boolean("panel-edit-mode", false);
@@ -519,6 +518,11 @@ function start() {
     });
 }
 
+function updateAnimationsEnabled() {
+    animations_enabled = !(software_rendering) && global.settings.get_boolean("desktop-effects-workspace");
+    cinnamonDBusService.notifyAnimationsEnabled();
+}
+
 function notifyCinnamon2d() {
     let icon = new St.Icon({ icon_name: 'driver-manager',
                              icon_type: St.IconType.FULLCOLOR,
@@ -1074,11 +1078,8 @@ function logStackTrace(msg) {
  * Returns (boolean): whether the window is on the workspace
  */
 function isWindowActorDisplayedOnWorkspace(win, workspaceIndex) {
-    if (win.get_workspace() == workspaceIndex) {return true;}
-    let mwin = win.get_meta_window();
-    return mwin && (mwin.is_on_all_workspaces() ||
-        (wm.workspacesOnlyOnPrimary && mwin.get_monitor() != layoutManager.primaryIndex)
-    );
+    return win.get_workspace() == workspaceIndex ||
+        (win.get_meta_window() && win.get_meta_window().is_on_all_workspaces());
 }
 
 /**
@@ -1103,6 +1104,8 @@ function getWindowActorsForWorkspace(workspaceIndex) {
 function _stageEventHandler(actor, event) {
     if (modalCount == 0)
         return false;
+    // log("Stage event handler........." + event.type() + "..." + event);
+
     if (event.type() != Clutter.EventType.KEY_PRESS) {
         if(!popup_rendering_actor || event.type() != Clutter.EventType.BUTTON_RELEASE)
             return false;
@@ -1115,9 +1118,8 @@ function _stageEventHandler(actor, event) {
 
     // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
     let action = global.display.get_keybinding_action(keyCode, modifierState);
-
-    if (action == Meta.KeyBindingAction.CUSTOM) {
-        global.display.keybinding_action_invoke_by_code(keyCode, modifierState);
+    if (action > 0) {
+        keybindingManager.invoke_keybinding_action_by_id(action);
     }
 
     // Other bindings are only available when the overview is up and no modal dialog is present
@@ -1201,7 +1203,7 @@ function pushModal(actor, timestamp, options) {
             log('pushModal: invocation of begin_modal failed');
             return false;
         }
-        Meta.disable_unredirect_for_screen(global.screen);
+        Meta.disable_unredirect_for_display(global.display);
     }
 
     global.set_stage_input_mode(Cinnamon.StageInputMode.FULLSCREEN);
@@ -1290,7 +1292,7 @@ function popModal(actor, timestamp) {
 
     layoutManager.updateChrome(true);
 
-    Meta.enable_unredirect_for_screen(global.screen);
+    Meta.enable_unredirect_for_display(global.display);
 }
 
 /**
@@ -1487,7 +1489,7 @@ function isInteresting(metaWindow) {
         return false;
 
     // Include any window the tracker finds interesting
-    if (metaWindow.is_interesting()) {
+    if (tracker.is_window_interesting(metaWindow)) {
         return true;
     }
 
@@ -1536,11 +1538,16 @@ function getTabList(workspaceOpt, screenOpt) {
 }
 
 function restartCinnamon(showOsd = false) {
-    if (showOsd) {
-        let dialog = new ModalDialog.InfoOSD(_("Restarting Cinnamon..."));
-        dialog.actor.add_style_class_name('restart-osd');
-        dialog.show();
-    }
+    global.display.connect("show-restart-message", () => {
+        if (showOsd) {
+            let dialog = new ModalDialog.InfoOSD(_("Restarting Cinnamon..."));
+            dialog.actor.add_style_class_name('restart-osd');
+            dialog.show();
+
+            return true;
+        }
+        return false;
+    });
 
     global.reexec_self();
 }
